# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-20 17:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FileRequest',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('criteria', models.TextField(blank=True)),
                ('filehandle', models.FileField(blank=True, null=True, upload_to='')),
                ('filename', models.CharField(blank=True, max_length=200)),
                ('description', models.TextField(blank=True)),
                ('file_type', models.CharField(blank=True, db_index=True, max_length=100)),
                ('content_type', models.CharField(blank=True, max_length=200)),
                ('size', models.BigIntegerField(blank=True, null=True)),
                ('requested_at', models.DateTimeField(db_index=True)),
                ('finished_at', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('expires_at', models.DateTimeField(blank=True, null=True)),
                ('requester', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='requested_downloads', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-requested_at'],
            },
        ),
    ]
